// Descriptive statistics

// from Scilab only
// ----------------
// q=quantile(x,al)      al-quantile of sample x
// m=median(x);          median of x
// m=mean(x)             average of x
// m=meanf(x,f)          average from values and frequencies
// v=variance(x)         sample variance of x
// v=variancef(x,f)      sam.var. from values and frequencies
// s=stdev(x)            sample standard deviation
// s=stdevf(x,f)         sam.stdev.  from values and frequencies
// cr=correl(x,y)        sample correlation coefficint
// q=iqr(x)              interquantile range
// r=strange(x)          range (max - min)
// m=moment(x,k)         non-central moment of oreder k
// m=cmoment(x,k)        central moment of oreder k
// histplot(n,x)         histogram of x with n columns
// from Scilsb Stixbox
// -------------------
// histo(x,n)            histogram of x with n columns
// boxplot(x(:))         box and whiskers plot
// qqplot(x,y)           quantile-quantile plot (comparison of distributions)

// another statistical functions
// -----------------------------
function mod=modus(x)
  v=vals(x);
  m=max(v(2,:));
  j=find(v(2,:)==m);
  mod=v(1,j);
endfunction

function c=covariance(x,y)
  // covariance of x,y ...  sum((x-mx).*(y-my))/(n-1)
  c=cov(x(:),y(:))(1,2);
endfunction

// Auxiliary functions
// ==================================================
function aa()
  // abort intereupted program (by pause)
  abort
endfunction

function cla            
  // close all figures             
  xdel(winsid())
endfunction

function c=comb(n,k)
  // combination number n!/(k'*(n-k)!)
  // computed in logarithms
  
  num=factL(n);
  den=factL(k)+factL(n-k);
  c=exp(num-den);
endfunction

function Lf=factL(n)
  // logarithm of factorial
  Lf=sum(log(1:n));
endfunction

function M=list2mat(L)
  // conversion of list to matrix
  n=length(L);
  for i=1:n
    M(:,i)=L(i)(:);
  end
endfunction

function L=mat2list(M)
  // list from columns of matrix
  n=size(M,2);
  L=list();
  for i=1:n
    L(i)=M(:,i);
  end
endfunction

function j=ranks(a)
  // ranks of a discrete random variable
  // j  ranks, i.e. for a=[3 5 2 2] we get j=[3 4 1.5 1.5]
  //    for equal values in a, we set average rank
  // a  vector of discrete (and maybe also real) values
  [u,i]=gsort(a,'g','i');
  [xx,j]=gsort(i,'g','i');

  av=u(1);
  for i=2:length(a)
    if abs(u(i-1)-u(i))>1e-8
      av=[av u(i)];
    end
  end
  for i=1:length(av)
    ia=find(abs(a-av(i))<1e-8)
    if length(ia)>1
      k=j(ia);
      j(ia)=mean(k);
    end
  end
endfunction

function s=samp(n,a)
  // sampling without repetition
  // n - sample length
  // a - set to be sampled
  m=length(a);
  for i=1:n
    j=fix(m*rand(1,1,'u'))+1;
    s(1,i)=a(j);
    a(j)=[];
    m=length(a);
  end
endfunction

function scatt(x,y) 
  // scatterplot
  if argn(2)>1, 
    if length(x)~=length(y)
      disp 'The vectors do not have the same length'
      return
    end    
    x=[x(:)';y(:)']; 
  end
  set(scf(),'position',[100 100 800 500]);
  plot(x(1,:),x(2,:),'b.')
endfunction

function T=table(x, y)
  // T=table(x,y)   contingency table T(nx,ny)
  // x,y       data
  
  nx = length(x);
  ny = length(y);
  nn = min(nx,ny);
  xx = vals(x);
  yy = vals(y);
  mx = size(xx,2);
  my = size(yy,2);
  T = zeros(mx, my);
  for t = 1:nn
    i=find(xx(1,:)==x(t));
    j=find(yy(1,:)==y(t));
    T(i, j) = T(i, j) + 1;
  end
endfunction

function [h,f]=vals(a)
  // [h f]=vals(a)  find different values of a variable
  //                and their frequencies               
  // h       values and frequencies  [vals;abs_freq]
  // f       relative frequencies
  
  a=a(:)';
  b=gsort(a,'g','i');
  [v,m]=unique(b);
  dm=diff(m);
  n1=length(b)+1;
  n=[dm n1-m($)];
  f=n/sum(n);
  h=[v(:)';n];

  if sum(n)~=max(size(a))
    disp('Error: in vals.sci')
    return
  end
endfunction
