function [lb, ub]=z_int (aver,vr,n,side,alpha)
    //ub ... upper border
    //lb ... lower border
    //aver ... sample average
    //vr ...  population variance
    //n ... sample length
    //side ... typ IS ('l'=left, 'r'=right, 'b' both)
    //alpha ... confidence
    if argn(2)==4 then
       alpha=0.05;
    end
    if alpha>0.5 then
        disp('set alpha < 0.5');
        return
    end

    select side
    case 'b' then
        crVal=cdfnor("X",0,1,1-alpha/2,alpha/2);
        ub=aver+crVal*sqrt(vr)/sqrt(n);
        lb=aver-crVal*sqrt(vr)/sqrt(n);
    case 'r' then
        crVal=cdfnor("X",0,1,1-alpha,alpha);
        ub=aver+crVal*sqrt(vr)/sqrt(n);
        lb=-%inf;
    case 'l' then
        crVal=cdfnor("X",0,1,1-alpha,alpha);
        ub=%inf;
        lb=aver-crVal*sqrt(vr)/sqrt(n);
    else
        disp('Variable side can take values ''l'', ''r'' or ''b''')
        lb=[]; ub=[]; return
    end
endfunction

function [lb, ub]=t_int (aver,vr,n,side,alpha)
    //ub ... upper border
    //lb ... lower border
    //aver ... sample average
    //vr ... vr výběru
    //n ... pocet prvků výběru/prvky výběru
    //side ... typ IS ('l'=left, 'r'=right, 'b' both)
    //alpha ... confidence

    if argn(2)==4 then
       alpha=0.05;
    end
    if alpha>0.5 then
        disp('set alpha < 0.5');
        return
    end
    
    select side
    case 'b' then
        crVal=cdft("T",n-1,1-alpha/2,alpha/2);
        ub=aver+crVal*sqrt(vr)/sqrt(n);
        lb=aver-crVal*sqrt(vr)/sqrt(n);
    case 'r' then
        crVal=cdft("T",n-1,1-alpha,alpha);
        ub=aver+crVal*sqrt(vr)/sqrt(n);
        lb=-%inf;
    case 'l' then
        crVal=cdft("T",n-1,1-alpha,alpha);
        ub=%inf;
        lb=aver-crVal*sqrt(vr)/sqrt(n);
    else
        disp('Variable side can take values ''l'', ''r'' or ''b''')
        lb=[]; ub=[]; return
    end   
endfunction

function [lb, ub]=prop_int(p,n,side,alpha)
    //ub ... upper border
    //lb ... lower border
    //p ... prob. of success
    //n ... sample length
    //side ... typ IS ('l'=left, 'r'=right, 'b' both)
    //alpha ... confidence
    if argn(2)==3 then
       alpha=0.05;
    end    
    if p>1 then
        prav=p/n;
    else
        prav=p;
    end
    if alpha>0.5 then
        disp('set alpha < 0.5');
        return
    end
    
    select side
    case 'b' then
        crVal=cdfnor("X",0,1,1-alpha/2,alpha/2);
        ub=prav+sqrt(prav*(1-prav)/n)*crVal;
        lb=prav-sqrt(prav*(1-prav)/n)*crVal;
    case 'r' then
        crVal=cdfnor("X",0,1,1-alpha,alpha);
        ub=prav+sqrt(prav*(1-prav)/n)*crVal;
        lb=0;
    case 'l' then
        crVal=cdfnor("X",0,1,1-alpha,alpha);
        ub=1;
        lb=prav-sqrt(prav*(1-prav)/n)*crVal;
    end
endfunction

function [lb, ub]=var_int(vr,n,side,alpha)
    //ub ... upper border
    //lb ... lower border
    //vr ... sample variance
    //n ... sample length
    //side ... typ IS ('l'=left, 'r'=right, 'b' both)
    //alpha ... confidence    
    if argn(2)==3 then
       alpha=0.05;
    end
    if alpha>0.5 then
        disp('set alpha < 0.5');
        return
    end
    select side
    case 'b' then
        crVal=cdfchi("X",n-1,alpha/2,1-alpha/2);
        dm_chi_alpha_2=cdfchi("X",n-1,1-alpha/2,alpha/2);
        ub=(n-1)*vr/crVal;
        lb=(n-1)*vr/dm_chi_alpha_2;
    case 'r' then
        crVal=cdfchi("X",n-1,alpha,1-alpha);
        ub=(n-1)*vr/crVal;
        lb=0;
    case 'l' then
        dm_chi_alpha_2=cdfchi("X",n-1,1-alpha,alpha);
        ub=%inf;
        lb=(n-1)*vr/dm_chi_alpha_2;
    else
        disp('Variable side can take values ''l'', ''r'' or ''b''')
        lb=[]; ub=[]; return
    end
endfunction

