//z_test
function pv=z_test(mu0,av,vr,n,side)
    //pv... p-value
    //mu0 ... expectation via H0
    //av ... sample average
    //vr ... population variance
    //n ... sample length    
    //side ... typ IS ('l'=left, 'r'=right, 'b' both)

    T=(av-mu0)/(sqrt(vr)/sqrt(n));
    [P,Q]=cdfnor("PQ",T,0,1);
    select side
    case 'b' then
        pv=2*min(P,Q);
    case 'r' then
        pv=Q;
    case 'l' then
        pv=P;
    else
        disp('Variable side can take values ''l'', ''r'' or ''b''')
        pv=[]; return
    end
endfunction

//t test
function pv=t_test(mu0,av,vr,n,side)
    //pv... p-value
    //mu0 ... expectation via H0
    //av ... sample average
    //vr ... sample variance
    //n ... sample length
    //side ... typ IS ('l'=left, 'r'=right, 'b' both)

    
    T=(av-mu0)/(sqrt(vr)/sqrt(n));
    [P,Q]=cdft("PQ",T,n-1)
    select side
    case 'b' then
        pv=2*min(P,Q)
    case 'r' then
        pv=Q;
    case 'l' then
        pv=P;
    else
        disp('Variable side can take values ''l'', ''r'' or ''b''')
        pv=[]; return
    end
endfunction

//test of ratio
function pv=prop_test(p0,p,n,side)
    //pv... p-value
    //p0 ... probab. via H0
    //p ... probab. from sample
    //n ... sample length
    //side ... typ IS ('l'=left, 'r'=right, 'b' both)

    if p>1 then
        prav=p/n;
    else
        prav=p;
    end

    T=(prav-p0)/sqrt(p0*(1-p0)/n)
    [P,Q]=cdfnor("PQ",T,0,1);
    select side
    case 'b' then
        pv=2*min(P,Q);
    case 'r' then
        pv=Q;
    case 'l' then
        pv=P;
    else
        disp('Variable side can take values ''l'', ''r'' or ''b''')
        pv=[]; return
    end
endfunction

//test of variance
function pv=var_test(vr0,vr,n,side)
    //pv... p-value
    //n ... sample length
    //vr ... sample variance
    //side ... typ IS ('l'=left, 'r'=right, 'b' both)

    T=((n-1)/vr0)*vr;
    [P,Q]=cdfchi("PQ",T,n-1)
    select side
    case 'b' then
        pv=2*min(P,Q);
    case 'r' then
        pv=Q;
    case 'l' then
        pv=P;
    else
        disp('Variable side can take values ''l'', ''r'' or ''b''')
        pv=[]; return
    end  
endfunction

// ------------

//parametric tests for two samples
function pv=z_test_2(av1,vr1,n1,av2,vr2,n2,side)
    //pv ... p-value
    //av1 ... sample average 1
    //vr1 ... sample variance 1
    //n1 .. sample length
    //av2 ... sample average 2
    //vr2 ... sample variance 2
    //n2 ... sample length 2
    //side ... typ IS ('l'=left, 'r'=right, 'b' both)

    T=(av1-av2)/sqrt((vr1/n1)+(vr2/n2));
    [P,Q]=cdfnor("PQ",T,0,1);
    select side
    case 'b' then
        pv=2*min(P,Q);
    case 'r' then
        pv=Q;
    case 'l' then
        pv=P;
    else
        disp('Variable side can take values ''l'', ''r'' or ''b''')
        pv=[]; return
    end
endfunction

//t test - equal variances
function pv=t_test_2s(av1,vr1,n1,av2,vr2, n2,side)
    //pv ... p-value
    //av1 ... sample average 1
    //vr1 ... sample variance 1
    //n1 .. sample length 1
    //av2 ... sample average 2
    //vr2 ... sample variance č. 2
    //n2 ... sample length č. 2
    //side ... typ IS ('l'=left, 'r'=right, 'b' both)

    Df=n1+n2-2;
    sp=((n1-1)*vr1+(n2-1)*vr2)/(Df);   
    T=(av1-av2)/(sqrt(sp)*sqrt((1/n1)+(1/n2)));
    [P,Q]=cdft("PQ",T,Df)
    select side
    case 'b' then
        pv=2*min(P,Q)
    case 'r' then
        pv=Q;
    case 'l' then
        pv=P;
    else
        disp('Variable side can take values ''l'', ''r'' or ''b''')
        pv=[]; return
    end
endfunction

//t test different variances
function pv=t_test_2n(av1,vr1,n1,av2,vr2, n2,side)
    //pv ... p-value
    //av1 ... sample average 1
    //vr1 ... sample variance 1
    //n1 .. sample length 1
    //av2 ... sample average 2
    //vr2 ... sample variance 2
    //n2 ... sample length 2
    //side ... typ IS ('l'=left, 'r'=right, 'b' both)

    
    R1_n1=vr1/n1;
    R2_n2=vr2/n2;   
    T=(av1-av2)/(sqrt(R1_n1+R2_n2));
    Df=round((R1_n1+R2_n2)^2/((R1_n1^2/(n1-1))+(R2_n2^2/(n2-1))));
    [P,Q]=cdft("PQ",T,Df)
    select side
    case 'b' then
        pv=2*min(P,Q)
    case 'r' then
        pv=Q;
    case 'l' then
        pv=P;
    else
        disp('Variable side can take values ''l'', ''r'' or ''b''')
        pv=[]; return
    end
endfunction

//t test paired
function pv=t_test_2p(sample1,sample2,side)
    //pv ... p-value
    //sample1 ... values of the sample 1
    //sample2 ... values of the sample 2
    //side ... typ IS ('l'=left, 'r'=right, 'b' both)

    vektor1=sample1(:)';
    vektor2=sample2(:)';

    n=max(size(vektor1));
    x_d=sum(vektor1-vektor2)/n;
    s_d=sqrt((sum((vektor1-vektor2)^2)/n)-x_d^2);
    T=(x_d/s_d)*sqrt(n-1);
    Df=n-1;
    [P,Q]=cdft("PQ",T,Df)
    select side
    case 'b' then
        pv=2*min(P,Q)
    case 'r' then
        pv=Q;
    case 'l' then
        pv=P;
    else
        disp('Variable side can take values ''l'', ''r'' or ''b''')
        pv=[]; return
    end
endfunction

//test of ratio
function pv=prop_test_2(p1,n1,p2,n2,side)
    //pv ... p-value
    //p1 ... sample probability 1
    //n1 ... sample length  1
    //p2 ... sample probability 2
    //n2 ... pocet prvků výběru 2
    //side ... typ IS ('l'=left, 'r'=right, 'b' both)


    if p1>1 then
        prav1=p1/n1;
    else
        prav1=p1;
    end
    if p2>1 then
        prav2=p2/n2;
    else
        prav2=p2;
    end

    pp=(prav1*n1+prav2*n2)/(n1+n2);
    T=(prav1-prav2)/(sqrt(pp*(1-pp))*sqrt(1/n1+1/n2));
    [P,Q]=cdfnor("PQ",T,0,1);
    select side
    case 'b' then
        pv=2*min(P,Q);
    case 'r' then
        pv=Q;
    case 'l' then
        pv=P;
    else
        disp('Variable side can take values ''l'', ''r'' or ''b''')
        pv=[]; return
    end
endfunction

//test two variances
function pv=var_test_2(vr1,n1,vr2,n2,side)
    //pv... p-value
    //vr1 ... sample variance 1
    //n1 ... sample length 1
    //vr2 ... sample variance 2
    //n1 ... sample length 2
    //side ... typ IS ('l'=left, 'r'=right, 'b' both)

    if vr1>vr2 then
        T=vr1/vr2;
    else
        T=vr2/vr1;
    end
    Df1=n1-1;
    Df2=n2-1;
    [P,Q]=cdff("PQ",T,Df1,Df2);

    select side
    case 'b' then
        pv=2*min(P,Q);
    case 'r' then
        pv=Q;
    case 'l' then
        pv=P;
    else
        disp('Variable side can take values ''l'', ''r'' or ''b''')
        pv=[]; return
    end  
endfunction


  
