//chi square tests
function pv=chisquare_test(O,E)
    //chi square test of homogeneity
    //pv ... p-value
    //O ... observed frequencies
    //E ... expected frequencies

    O=O(:)';
    E=E(:)';
    n=size(O,2);
    
    T=sum((O-E).^2 ./E);
    [P,Q]=cdfchi("PQ",T,n-1)
    pv=Q;
endfunction

function pv=chisquare_test_i(KT)
    //chi square test of independence
    //pv ... p-value
    //KT ... contingency tabulka
    
    [r,s]=size(KT);
    pr_KT=KT/sum(KT);
    pr_O=sum(pr_KT,2)*sum(pr_KT,1);
    O=pr_O*sum(KT);
    
    T=sum((KT-O).^2 ./O);  
    Df=(r-1)*(s-1);
    [P,Q]=cdfchi("PQ",T,Df)
    pv=Q;
endfunction

//sign test
function pv=sign_test(X,Y,side)
    //pv ... p-value
   //X ... random sample 1
    //Y ... random sample 2 / median
    //side ... typ ('l'=left, 'r'=right, 'b' both)

    X=X(:)';
    Y=Y(:)';
    
    if size(Y,2)==1 
        disp('medianovy test')
    end
    n=size(X,2);
    b=sum(X>Y);
    T=(2*b-n)/sqrt(n);
    [P,Q]=cdfnor("PQ",T,0,1);
    select side
    case 'b' 
        pv=2*min(P,Q);
    case 'r' 
        pv=Q;
    case 'l' 
        pv=P;
    else
        disp('Variable side can take values ''l'', ''r'' or ''b''')
        pv=[]; return    end
endfunction
    
//Pearson test
function pv=pearson_test(X,Y) 
    //pv ... p-value
    //X ... random sample 1
    //Y ... random sample 2

    X=X(:)';
    Y=Y(:)';

    r=covariance(X,Y)/sqrt(variance(X)*variance(Y));
    n=size(X,2);
    T=r/sqrt((1-r^2)/(n-2));
    Df=n-2;
    [P,Q]=cdft("PQ",T,Df)
    pv=2*min(P,Q)
endfunction

//Spearman test
function pv=spearman_test(x,y)
  // Spearman test
  //pv ... p-value
  //x ... random sample 1
  //y ... random sample 2
  
  xr=ranks(x);
  yr=ranks(y);
  
  pv=pearson_test(xr,yr);
endfunction

// ----------------

// Mann-Whitney test 
function pv=mannwhit_test(x1,x2)
  // x1,x2     independent samples 
  // for U<Kr reject - pv=0 (otherwise pv=1)

  tab7=[...
  %nan %nan %nan %nan %nan %nan %nan %nan %nan %nan %nan %nan %nan %nan %nan %nan %nan %nan %nan %nan 
  %nan %nan %nan %nan %nan %nan %nan 0 0 0 0 1 1 1 1 1 2 2 2 2 
  %nan %nan %nan %nan 0 1 1 2 2 3 3 4 4 5 5 6 6 7 7 8 
  %nan %nan %nan 0 1 2 3 4 4 5 6 7 8 9 10 11 11 12 13 13 
  %nan %nan 0 1 2 3 5 6 7 8 9 11 12 13 14 15 17 18 19 20 
  %nan %nan 1 2 3 5 6 8 10 11 13 14 16 17 19 21 22 24 25 27 
  %nan %nan 1 3 5 6 8 10 12 14 16 18 20 22 24 26 28 30 32 34 
  %nan 0 2 4 6 8 10 13 15 17 19 22 24 26 29 31 34 36 38 41 
  %nan 0 2 4 7 10 12 15 17 21 23 26 28 31 34 37 39 42 45 48 
  %nan 0 3 5 8 11 14 17 20 23 26 29 33 36 39 42 45 48 52 55 
  %nan 0 3 6 9 13 16 19 23 26 30 33 37 40 44 47 51 55 58 62 
  %nan 1 4 7 11 14 18 22 26 29 33 37 41 45 49 53 57 61 65 69 
  %nan 1 4 8 12 16 20 24 28 33 37 41 45 50 54 59 63 67 72 76 
  %nan 1 5 9 13 17 22 26 31 36 40 45 50 55 59 64 67 74 78 83 
  %nan 1 5 10 14 19 24 29 34 39 44 49 54 59 64 70 75 80 85 90 
  %nan 1 6 11 15 21 26 31 37 42 47 53 59 64 70 75 81 86 92 98 
  %nan 2 6 11 17 22 28 34 39 45 51 57 63 67 75 81 87 93 99 105 
  %nan 2 7 12 18 24 30 36 42 48 55 61 67 74 80 86 93 99 106 112 
  %nan 2 7 13 19 25 32 38 45 52 58 65 72 78 85 92 99 106 113 119 
  %nan 2 8 14 20 27 34 41 48 55 62 69 76 83 90 98 105 112 119 127
  ];

  n1=length(x1);
  n2=length(x2);

  x=[x1 x2];
  r=ranks(x);

  r1=r(1:n1);
  r2=r((n1+1):(n1+n2));

  s1=sum(r1);
  s2=sum(r2);

  U1=n1*n2+n1*(n1+1)/2-s1
  U2=n1*n2+n2*(n2+1)/2-s2
  U=min([U1 U2])

  if max(n1,n2)<=20
    disp 'Mann Whitney: small sample'
    Kr=tab7(n1,n2)
    if U>Kr
      pv=1
    else
      pv=0;
    end
  else
    // for n >> normal approximation
    Z=(U-n1*n2/2)/sqrt(n1*n2*(n1+n2+1)/12);
    [P,Q]=cdfnor("PQ",Z,0,1);
    pv=2*min(P,Q);
  end
endfunction

// Wilcoxon test
function pv=wilcoxon_test(x1,x2,side)
  // Only for confidence level = 0.05 !!!
  // pv   p-value - for n>20
  // W    statistics - for n<=20
  // kr1,kr2   critical values
  // H0: both: E[D]=0, left: E[D]>0, right: E[D]<0
  //     where D is x1-x2 
  // H0 is rejected if  W is outside of (kr1,kr2)
  // x1,x2     samples

  if argn(2)<3, side='b'; end
  if length(x2)==1
    x2=x2*ones(x1);
  end
  if length(x1)==1
    x1=x1*ones(x2);
  end
  pv=%nan; W=%nan; kr=%nan;   // emergency case
//al=[0.05 0.01 0.001];       // zatím jen pro al=0.05

Tb=[..
0 0 0
0 0 0
0 0 0
0 0 0
0 0 0
0.6 0 0
2.1 0 0
3.7 0.3 0
5.5 1.6 0
8.1 3.1 0
10.7 5.1 0
13.8 7.2 1.0
17.2 9.8 2.5
21.1 12.7 4.4
25.3 15.9 6.5
29.9 19.5 9.0
34.9 23.4 11.7
40.3 27.7 14.8
46.1 32.4 18.2
52.3 37.4 21.9];
Tab=Tb(:,1);

  if length(x1)~=length(x2)
    disp "Wilcoxon test: the samples must be of the same length";
    disp ' '
    pv=%nan; W=%nan; kr=%nan;
    return
  end

  x=x1-x2;
  D=[];
  s=[];
  for i=1:length(x)
    if abs(x(i))>1e-8
      s=[s sign(x(i))];
      D=[D abs(x(i))];
    end
  end
  n=length(D);
  R=ranks(D);

//  n_minus=length(find(s==-1));          // test for plus diferences
  jk=find(s==1);                        // for criterion
//   n_plus=length(jk);                   // test for minus diferences
//  if min(n_minus,n_plus)<5
//    printf('\n Wilcoxon test: needs at least 5 + and - differences\n\n');
//    return
//  end
  W=sum(R(jk))

  if n>20
    m=n*(n+1)/4;
    s=sqrt(n*(n+1)*(2*n+1)/24);
    Z=(W-m)/s;
    [P,Q]=cdfnor("PQ",Z,0,1);
    select side
    case 'r'   // pravo
      pv=Q
    case 'l'   // levo
      pv=P
    case 'b'   // obou
      pv=2*min([P Q]);
    end
  else 
    disp 'Wicoxon: small sample'    
    kr=Tb(n,1);     // for al=0.05
    if W<=kr
      pv=0;         // = reject
    else
      pv=1;         // = do not reject
    end
  end
endfunction

// McNemar test
function pv=mcnemar_test(Tab)
  // Tab ... table
  // pv ... p-value
  
  T=(Tab(1,2)-Tab(2,1))^2/(Tab(1,2)+Tab(2,1));
  [P,Q]=cdfchi("PQ",T,1);
  pv=Q
endfunction 

// simple test of normality
function pv=norm_test(x,al)
  // x    sample
  // al   significance  [0 .005 .01 .025 .05 .1]
  // q    statistics
  // krD  lower border
  // krH  upper border
  // If q is outside interval (krD, krH)  normality is rejected

  tab14=[...
  3 1.732 2.000 1.735 2.000 1.737 2.000 1.745 2.000 1.758 1.999 1.782 1.997
  4 1.732 2.449 1.82 2.447 1.87 2.445 1.93 2.439 1.98 2.429 2.04 2.409
  5 1.826 2.828 1.98 2.813 2.02 2.803 2.09 2.782 2.15 2.753 2.22 2.712
  6 1.826 3.162 2.11 3.115 2.15 3.095 2.22 3.056 2.28 3.012 2.37 2.949
  7 1.871 3.464 2.22 3.369 2.26 3.338 2.33 3.282 2.40 3.222 2.49 3.143
  8 1.871 3.742 2.31 3.585 2.35 3.543 2.43 3.471 2.50 3.399 2.59 3.308
  9 1.897 4.000 2.39 3.772 2.44 3.720 2.51 3.634 2.59 3.552 2.68 3.449
  10 1.897 4.243 2.46 3.935 2.51 3.875 2.59 3.777 2.67 3.685 2.76 3.57
  11 1.915 4.472 2.53 4.079 2.58 4.012 2.66 3.903 2.74 3.80 2.84 3.68
  12 1.915 4.690 2.59 4.208 2.64 4.134 2.72 4.02 2.80 3.91 2.90 3.78
  13 1.927 4.899 2.64 4.325 2.70 4.244 2.78 4.12 2.86 4.00 2.96 3.87
  14 1.927 5.099 2.70 4.431 2.75 4.34 2.83 4.21 2.92 4.09 3.02 3.95
  15 1.936 5.292 2.74 4.53 2.80 4.44 2.88 4.29 2.97 4.17 3.07 4.02
  16 1.936 5.477 2.79 4.62 2.84 4.52 2.93 4.37 3.01 4.24 3.12 4.09
  17 1.944 5.657 2.83 4.70 2.88 4.60 2.97 4.44 3.06 4.31 3.17 4.15
  18 1.944 5.831 2.87 4.78 2.92 4.67 3.01 4.51 3.10 4.37 3.21 4.21
  19 1.949 6.000 2.90 4.85 2.96 4.74 3.05 4.56 3.14 4.43 3.25 4.27
  20 1.949 6.164 2.94 4.91 2.99 4.80 3.09 4.63 3.18 4.49 3.29 4.32
  25 1.961 6.93 3.09 5.19 3.15 5.06 3.24 4.87 3.34 4.71 3.45 4.53
  30 1.966 7.62 3.21 5.40 3.27 5.26 3.37 5.06 3.47 4.89 3.59 4.70
  35 1.972 8.25 3.32 5.57 3.38 5.42 3.48 5.21 3.58 5.04 3.70 4.84
  40 1.975 8.83 3.41 5.71 3.47 5.56 3.57 5.34 3.67 5.16 3.79 4.96
  45 1.978 9.38 3.49 5.83 3.55 5.67 3.66 5.45 3.75 5.26 3.88 5.06
  50 1.980 9.90 3.56 5.93 3.62 5.77 3.73 5.54 3.83 5.35 3.95 5.14
  55 1.982 10.39 3.62 6.02 3.69 5.86 3.80 5.63 3.90 5.43 4.02 5.22
  60 1.983 10.86 3.68 6.10 3.75 5.94 3.86 5.70 3.96 5.51 4.08 5.29
  65 1.985 11.31 3.74 6.17 3.80 6.01 3.91 5.77 4.01 5.57 4.14 5.35
  70 1.986 11.75 3.79 6.24 3.85 6.07 3.96 5.83 4.06 5.63 4.19 5.41
  75 1.987 12.17 3.83 6.30 3.90 6.13 4.01 5.88 4.11 5.68 4.24 5.46
  80 1.987 12.57 3.88 6.35 3.94 6.18 4.05 5.93 4.16 5.73 4.28 5.51
  85 1.988 12.96 3.92 6.40 3.99 6.23 4.09 5.98 4.20 5.78 4.33 5.56
  90 1.989 13.34 3.96 6.45 4.02 6.27 4.13 6.03 4.24 5.82 4.36 5.60
  95 1.990 13.71 3.99 6.49 4.06 6.32 4.17 6.07 4.27 5.86 4.40 5.64
  100 1.990 14.07 4.03 6.53 4.10 6.36 4.21 6.11 4.31 5.90 4.44 5.68
  150 1.993 17.26 4.32 6.82 4.38 6.64 4.48 6.39 4.59 6.18 4.72 5.96
  200 1.995 19.95 4.53 7.01 4.59 6.84 4.68 6.60 4.78 6.39 4.90 6.15
  500 1.998 31.59 5.06 7.60 5.13 7.42 5.25 7.15 5.47 6.94 5.49 6.72
  1000 1.999 44.70 5.50 7.99 5.57 7.80 5.68 7.54 5.79 7.33 5.92 7.11
  ];
  Tal=[0 .005 .01 .025 .05 .1];

  Tn=tab14(:,1);

  [xxx mi]=min(abs(Tn-length(x)));
  n=Tn(mi);

  j=0;
  for i=2:2:13
    j=j+1;
    kr(j).a=tab14(:,i);
    kr(j).b=tab14(:,i+1);
  end

  it=find(Tn==n);
  jt=find(Tal==al);

  krD=kr(jt).a(it)
  krH=kr(jt).b(it)

  s=stdev(x);
  w=max(x)-min(x);
  q=w/s
  if (krD<q)&(q<krH)
    pv=1;
  else
    pv=0;
  end
endfunction

// shapiro test of normality
function pv=shapiro_test(x)
  // test of normality
  // x    sample
  // W    statistics
  // pv   p-value (reduced to 0, 0.01 0.05, 1)
  
  x=x(:)';
  t1=[%nan,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38
  1,0.6646,0.6431,0.6233,0.6052,0.5888,0.5739,0.5601,0.5475,0.5359,0.5251,0.5150,0.5056,0.4968,0.4886,0.4808,0.4734,0.4643,0.4590,0.4542,0.4493,0.4450,0.4407,0.4366,0.4328,0.4291,0.4254,0.4220,0.4188,0.4156,0.4127,0.4096,0.4068,0.4040,0.4015;
  2,0.2413,0.2806,0.3031,0.3164,0.3244,0.3291,0.3315,0.3325,0.3325,0.3318,0.3306,0.3290,0.3273,0.3253,0.3232,0.3211,0.3185,0.3156,0.3126,0.3098,0.3069,0.3043,0.3018,0.2992,0.2968,0.2944,0.2921,0.2898,0.2876,0.2854,0.2834,0.2813,0.2794,0.2774;
  3,0.0000,0.0875,0.1401,0.1743,0.1976,0.2141,0.2260,0.2347,0.2412,0.2460,0.2495,0.2521,0.2540,0.2553,0.2561,0.2565,0.2578,0.2571,0.2563,0.2554,0.2543,0.2533,0.2522,0.2510,0.2499,0.2487,0.2475,0.2463,0.2451,0.2439,0.2427,0.2415,0.2403,0.2391;
  4,%nan,%nan,0.0000,0.0561,0.0947,0.1224,0.1429,0.1586,0.1707,0.1802,0.1878,0.1939,0.1988,0.2027,0.2059,0.2085,0.2119,0.2131,0.2139,0.2145,0.2148,0.2151,0.2152,0.2151,0.2150,0.2148,0.2145,0.2141,0.2137,0.2132,0.2127,0.2121,0.2116,0.2110;
  5,%nan,%nan,%nan,%nan,0.0000,0.0399,0.0695,0.0922,0.1099,0.1240,0.1353,0.1447,0.1524,0.1587,0.1641,0.1686,0.1736,0.1764,0.1787,0.1807,0.1822,0.1836,0.1848,0.1857,0.1864,0.1870,0.1874,0.1878,0.1880,0.1882,0.1883,0.1883,0.1883,0.1881;
  6,%nan,%nan,%nan,%nan,%nan,%nan,0.0000,0.0303,0.0539,0.0727,0.0880,0.1005,0.1109,0.1197,0.1271,0.1334,0.1399,0.1443,0.1480,0.1512,0.1539,0.1563,0.1584,0.1601,0.1616,0.1630,0.1641,0.1651,0.1660,0.1667,0.1673,0.1678,0.1683,0.1686;
  7,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,0.0000,0.0240,0.0433,0.0593,0.0725,0.0837,0.0932,0.1013,0.1092,0.1150,0.1201,0.1245,0.1283,0.1316,0.1346,0.1372,0.1395,0.1415,0.1433,0.1449,0.1463,0.1475,0.1487,0.1496,0.1505,0.1513;
  8,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,0.0000,0.0196,0.0359,0.0496,0.0612,0.0711,0.0804,0.0878,0.0941,0.0997,0.1046,0.1089,0.1128,0.1162,0.1192,0.1219,0.1243,0.1265,0.1284,0.1301,0.1317,0.1331,0.1344,0.1356;
  9,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,0.0000,0.0163,0.0303,0.0422,0.0530,0.0618,0.0696,0.0764,0.0823,0.0876,0.0923,0.0965,0.1002,0.1036,0.1066,0.1093,0.1118,0.1140,0.1160,0.1179,0.1196,0.1211;
  10,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,0.0000,0.0140,0.0263,0.0368,0.0459,0.0539,0.0610,0.0672,0.0728,0.0778,0.0822,0.0862,0.0899,0.0931,0.0961,0.0988,0.1013,0.1036,0.1056,0.1075;
  11,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,0.0000,0.0122,0.0228,0.0321,0.0403,0.0476,0.0540,0.0598,0.0650,0.0697,0.0739,0.0777,0.0812,0.0844,0.0873,0.0900,0.0924,0.0947;
  12,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,0.0000,0.0107,0.0200,0.0284,0.0358,0.0424,0.0483,0.0537,0.0585,0.0629,0.0669,0.0706,0.0739,0.0770,0.0798,0.0824;
  13,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,0.0000,0.0094,0.0178,0.0253,0.0320,0.0381,0.0435,0.0485,0.0530,0.0572,0.0610,0.0645,0.0677,0.0706;
  14,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,0.0000,0.0084,0.0159,0.0227,0.0289,0.0344,0.0395,0.0441,0.0484,0.0523,0.0559,0.0592;
  15,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,0.0000,0.0076,0.0144,0.0206,0.0262,0.0314,0.0361,0.0404,0.0444,0.0481;
  16,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,0.0000,0.0068,0.0131,0.0187,0.0239,0.0287,0.0331,0.0372;
  17,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,0.0000,0.0062,0.0119,0.0172,0.0220,0.0264;
  18,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,0.0000,0.0057,0.0110,0.0158;
  19,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,%nan,0.0000,0.0153];

  pvs=[.05 .01]
  t2=[5,0.762,0.686;
  6,0.988,0.713;
  7,0.803,0.730;
  8,0.818,0.749;
  9,0.829,0.764;
  10,0.842,0.781;
  11,0.850,0.792;
  12,0.859,0.805;
  13,0.866,0.814;
  14,0.874,0.825;
  15,0.881,0.835;
  16,0.887,0.844;
  17,0.892,0.851;
  18,0.897,0.858;
  19,0.901,0.863;
  20,0.905,0.868;
  21,0.908,0.873;
  22,0.911,0.878;
  23,0.914,0.881;
  24,0.916,0.884;
  25,0.918,0.888;
  26,0.920,0.891;
  27,0.923,0.894;
  28,0.924,0.896;
  29,0.926,0.898;
  30,0.927,0.900;
  31,0.929,0.902;
  32,0.930,0.904;
  33,0.931,0.906;
  34,0.933,0.908;
  35,0.934,0.910;
  36,0.935,0.912;
  37,0.936,0.914;
  38,0.938,0.916;
  39,0.939,0.917;
  40,0.940,0.919;
  41,0.941,0.920;
  42,0.942,0.922;
  43,0.943,0.923;
  44,0.944,0.924;
  45,0.945,0.926;
  46,0.945,0.927;
  47,0.946,0.928;
  48,0.947,0.929;
  49,0.947,0.929;
  50,0.947,0.930];

  n=length(x);
  if n>38
    disp 'Error: sample length <= 38';
    disp ' ';
    W=%nan; pv=%nan;
    return
  end
 
  r=gsort(x,'g','i');
  ss=sum((r-mean(r))^2);
  m=fix(n/2);
  r1=r(1:m);
  r2=r(n:-1:(n-m+1));
  a=t1(2:(m+1),n-3);
  b=sum(a.*(r2(:)-r1(:)));
  W=b^2/ss;
  pp=t2(n,2:$);
  d=abs(pp-W);
  [xxx,ipv]=min(d);
  pv=pvs(ipv);
  if W>pp(1)
    pv=1;
  end
  if W<pp(2)
    pv=0;
  end
endfunction


// chi-square test of normality
function pv=normCh2_test(x,n)
  // x    sample
  // n    number of intervals

  if argn(2)<2, n=10; end
  
  nx=length(x);
  mi=min(x);
  ma=max(x);
  mm=(ma-mi)/n;
  xi=mi:mm:ma;
  mx=mean(x);
  s2x=variance(x);

  o10=ones(1,n+1);
  F=cdfnor("PQ",xi,mx*o10,sqrt(s2x)*o10)*nx;


  E=diff(F);
  O=histc(xi,x,normalization=%f);

  ch2=sum(((O-E)^2)./E);
  df=n-1;

  [P,Q]=cdfchi("PQ",ch2,df);
  pv=Q;
endfunction

// Bartlett test
function pv=bartlett_test(L)
  // L  list of samples of matrix with samples in columns

  if type(L)~=15
    LM=mat2list(L);
    L=LM;
  end
  s1=0; s2=0; s3=0;
  a=max(size(L));
  S=zeros(1,a);
  n=0;
  for i=1:a
    ni=length(L(i));
    si=variance(L(i));

    s1=s1+1/(ni-1);
    s2=s2+(ni-1)*log(si);

    s3=s3+(ni-1)*si
    S(i)=si;
    n=n+ni;
  end
  C=1+(s1-1/(n-a))/(3*(a-1));
  s2c=(n-a)*log(s3/(n-a));
  B=(s2c-s2)/C;
  [P,Q]=cdfchi("PQ",B,a-1);
  pv=Q;
endfunction

// Kruskal-Wallis test
function pv=kruskal_test(L)
  // L  - list of samples (columns if it is matrix)
  
  d=[];
  k=max(size(L))

  for i=1:k
    n(i)=length(L(i));
    d=[d L(i)(:)'];
  end

  rk=ranks(d);

  r=list();
  n1=1;
  n2=n(1);
  for i=1:k
    r(i)=rk(n1:n2);
    t(i)=sum(r(i));
    if i<k
      n1=n2+1;
      n2=n2+(n(i+1));
    end
  end

  m=sum(n);
  T=12/(m*(m+1))*sum(t^2./n)-3*(m+1);
  [P,Q]=cdfchi("PQ",T,k-1)
  pv=Q;
endfunction

// Friedman test
function pv=friedman_test(x)
  // x matrix of data with samples in columns / bocks in rows 
  // i.e.:      shop1, shop2, shop3, shop4
  // inspector1    x     x      x      x
  // inspector2    x     x      x      x
  //     ... we compare shops, reviewers are not of interest
  //   block is a factor which is not of our interest
  //   different strictness if inspectors is compensated (blocked)
  
  [m,n]=size(x);           // počet řádků / sloupců (bloky)
  r=[];
  for i=1:m
    r=[r; ranks(x(i,:))];  // pořadí v řádcích 
  end
  for j=1:n
    R(j)=sum(r(:,j));      // součty pořadí ve sloupcích
  end
  // statistika
  T=12/(m*n*(n+1))*R'*R-3*m*(n+1)
  // kritický obor
  Df=n-1;
//  h=cdfchi("X",Df,.95,.05);
//  W=[h, %inf]
  // p-value
  pv=1-cdfchi("PQ",T,Df)
endfunction  
//
// Scheffé test
function T=scheffe_test(x,al)
  // L    list with samples or matrix with samples in columns
  // T=1 if the samples are different
  // it is a table for all combinations of variables
  
  // this is anova1
  if type(x)~=15
    Lm=mat2list(x);
    x=Lm;
  end

  nc=length(x);
  for i=1:nc
    n(i)=max(size(x(i)));
  end
  df=n-1;
  s=0;
  for i=1:nc
    mx(i)=mean(x(i));
    s=s+n(i)*mx(i);
  end
  xx=s/sum(n);
  ssb=0; ssw=0;
  for i=1:nc
    ssb=ssb+n(i)*(mx(i)-xx)^2;
    ssw=ssw+df(i)*variance(x(i));
  end
  ssc=ssb+ssw;
  dfb=nc-1;
  dfw=sum(df);
  dfc=dfb+dfw;
  msb=ssb/dfb;
  msw=ssw/dfw;
  MSe=msb/msw;      // this is the result of anova1
  // here ends the anova1
  // abd continues the sceffe test
  dF=cdff("F",dfb,dfw,1-al,al);
  for j=1:nc
    for k=j:nc
      M(j,k)=abs(mx(j)-mx(k));
      F(j,k)=sqrt(msw*dF*(nc-1)*(1/n(i)+1/n(j)));
    end
  end
  T=double(M>F); // the upper triangle is relevant
                    // (1 denostes the different variables)
endfunction  

// test for autoregression
function res=autoreg_test(x,y)
  // res  result
  // a    regression coefficient
  // x,y  data
  
  nd=length(y);
  X=[x(:) ones(nd,1)];
  Y=y(:);

  th=inv(X'*X)*X'*Y;
  yp=X*th;

  e=y(:)-yp;

  XX=e(1:nd-1);
  YY=e(2:nd);

  a=inv(XX'*XX)*XX'*YY
  if abs(a)<.3
    res='no autoregression';
  else
    res='autoregression detected';
  end
endfunction


