//linear regression
function p=lin_reg(x,y)
    //p ... parameters [b1 b0]
    //x ... independent varaible
    //y ... dependent variable
    x=x(:)';
    y=y(:)';
    n=length(x);
    if length(y) ~= n, 
        disp('vectors must be of the same length');
    end
    x=matrix(x,n,1);
    y=matrix(y,n,1);
    D=[y x ones(n,1)];
    V=D'*D;
    m=size(V,2);
    Vyf=V(2:m,1);
    Vf=V(2:m,2:m);
    p=inv(Vf)*Vyf;
endfunction  

//linear prediction
function yp=lin_pred(x,p)
    //yp ... prediction 
    //x ... independent varaible
    //p ... parameters [b1 b0]
    x=x(:)';
    m=length(p);
    yp=p(1)*x+p(2);
endfunction 

//multivariate linear regression
function p=lin_reg_n(x,y)
    //p ... parameters [bn bn-1 ... b1 b0]
    //x ... independent varaible
    //y ... dependent variable
    if size(x,1)<size(x,2) then
        x=x';
    end
    y=y(:);
    n=size(y,1);
    if size(x,1) ~= n 
        disp(' size(x,1) and length(y) must be the same');
        p=[];
        return
    end
    D=[x ones(n,1)];
    V=D'*D;
    p=inv(V+1e-8*eye(V))*D'*y
endfunction 

//multivariate linear prediction
function yp=lin_pred_n(x,p)
    //yp ... prediction 
    //x ... independent varaible
    //p ... parameters [bn bn-1 ... b1 b0]
    k=length(p);
    p=p(:);
    [m,n]=size(x);
    if n+1~=k
      disp 'Variables must be columns of x'
      yp=[];
      return
    end
    x1=[x ones(size(x,1),1)];
    yp=(x1*p)';
endfunction  

//exponential regression
function p=exp_reg(x,y)
    //p ... parameters [bn bn-1 ... b1 b0]
    //x ... independent varaible
    //y ... dependent variable
    n=max(size(x));
    if max(size(y)) ~= n, 
        disp('vectors must be of the same length');
    end
    x=matrix(x,n,1);
    y=matrix(y,n,1);
    x=matrix(x,n,1);
    if sum(y<0)>0
        disp('ERROR in exp_reg: There is a negative element in y!');
        return
    end
    D=[log(y) x ones(n,1)];
    V=D'*D;
    m=max(size(V));
    Vyf=V(2:m,1);
    Vf=V(2:m,2:m);
    p=inv(Vf)*Vyf;
    p(2)=exp(p(2));
endfunction  

//exponential prediction
function yp=exp_pred(x,p)
    //yp ... prediction
    //x ... independent varaible
    //p ... parameters [b1 b0]
    m=length(p);
    yp=exp(p(1)*x)*p(2);
endfunction  

//polynomial regression
function p=pol_reg(x,y,k)
    //p ... parameters [bn bn-1 ... b1 b0]
    //x ... independent varaible
    //y ... dependent variable
    //k ... stupeň regresního polynomu 
    n=length(x);
    if length(y) ~= n, 
        disp('vectors must be of the same length');
    end
    x=matrix(x,n,1);
    y=matrix(y,n,1);
    D=y;
    for i=1:k
        kk=k-i+1;
        D=[D x.^kk];
    end
    D=[D ones(n,1)];
    V=D'*D;
    m=max(size(V));
    Vyf=V(2:m,1);
    Vf=V(2:m,2:m);
    p=inv(Vf)*Vyf;
endfunction 

//polynomial prediction
function yp=pol_pred(x,p)
    //yp ... prediction závisle proměnné
    //x ... independent varaible
    //p ... parameters [b1 b0]
    m=max(size(p));
    yp=zeros(size(x,1),size(x,2));
    for i=1:m
        yp=yp.*x+p(i);
    end
endfunction

// -------------

//regression _ 2
function [b1,b0,r]=reg_desc(x,y)
    //b1 ... slope
    //b0 ... absolute term
    //r ... correl. coefficient
    //x ... independent varaible
    //y ... dependent variable
    n=max(size(x));
    if length(y)~=n, 
        disp('wrong lengths of x and y');
        return
    end
    mx=mean(x);
    my=mean(y);
    s2x=variance(x)*(n-1);
    s2y=variance(y)*(n-1);
    sxy=covariance(x,y)*(n-1);
    b1=sxy/s2x;
    b0=my-b1*mx;
    r=sxy/sqrt(s2x*s2y);
    yp=b1*x+b0;
endfunction

//f-test regression
function pv=f_test_reg(x,y)
    //pv... p-value
    //T ... statistika náhodné veličiny
    //df_num ... stupně volnosti pro čitatele
    //df_num ... stupně volnosti pro jmenovatele
    //x ... independent varaible
    //y ... dependent variable

    n=max(size(x));
    if length(y)~=n
        disp(' Chybne delky vektoru x a y');
        return
    end
    [b1,b0]=reg_desc(x,y);
    yp=b1*x+b0;
    my=mean(y);
    s1=sum((yp-my).^2);       // rozptyl vysvìtlený regresí
    s2=sum((y-yp).^2)/(n-2);  // nevysvětlený rozptyl 
    if s2<1e-8, s2=1e-8; end  //if
    T=s1/s2;
    df_num=1;
    df_den=n-2;
    [P,pv]=cdff("PQ",T,df_num,df_den);
endfunction  

//f test prediction
function pv=f_test_pred(y,yp,np)
    //pv... p-value
    //y ... dependent variable
    //yp ... prediction
    //np ... number of parameters
    if argn(2)<3, np=2; end
    n=length(y);
    my=mean(y);
    s1=sum((yp-my).^2)/(np-1); // rozptyl vysvětlený regresí
    s2=sum((y-yp).^2)/(n-np);  // nevysvětlený rozptyl 
    if s2<1e-8, 
        disp('Rozptyl  s2  je nula - vse je vysvetleno');
        s2=1e-8;
    end
    T=s1/s2;
    df_num=np-1;
    df_den=n-np;
    [P,pv]=cdff("PQ",T,df_num,df_den);
endfunction  

//test of independence of sample items
function pv=ordinal_test(x,al) 
    //pv ... p-value
    //X ... random sample
    //al ... confidence
    if argn(2)==1 
        al=0.05;
    end
    if al>0.5 
        disp('al < 0.5');
        return
    end
    x=x(:)';
    me_x=median(x);
    n=size(x,2);
    rozdil=x-me_x;
    b=1
    rozdil_bez_0=rozdil(rozdil~=0);
    
    for i=2:size(rozdil_bez_0,2)
        if rozdil_bez_0(i-1)*rozdil_bez_0(i)<0 
            b=b+1;
        end
    end
    T=(2*b-(n-2))/sqrt(n-1);
    [P,Q]=cdfnor("PQ",T,0,1);
    z_alpha=cdfnor("X",0,1,al,1-al);
    pv=P;
endfunction



