//Teoretické četnosti z kontingenční tabulky
function E=Teor_freq(KT)
    //Část chí kvadrát testu nezávislosti
    //Počítá teoretické četnosti z kontingenční tabulky
    //KT....kontingenční tabulka
    //E.....teoretické četnosti
    probability_KT=KT/sum(KT);
    probability_E=sum(probability_KT,2)*sum(probability_KT,1);
    E=probability_E*sum(KT);
endfunction


//Slučování řádků
function [KT,Boundary,Error]=merging_rows(Frequencies)
    //Příprava na Chí-kvadrát test 
    //Slučování radku tak, aby všechny očekávané teoretické četnosti byly
    //větší než 2
    //a aspoň 80% z očekávaných teoretických hodnot bylo větší než 5.
    //
    //Frequencies...Původní podrobná kontingenční tabulka
    //KT.........Výsledná sloučená kontingenční tabulka
    //Boundary....Hraniční řádky, které byly sloučeny vjedno
        //Např. Boundary=[1,5,10]
        //Tedy první řádek obsahuje 1. až 4. původní řádek,
        //druhý 5. až 9. původní řádek,
        //třetí 10. až poslední řádek.
    //Error .....Logická proměnná. Když je True, nepodařilo se vytvořit 
        //kontingenční tabulku tak, aby měla aspoň dva řádky, všechny 
        //očekávané teoretické četnosti byly větší než 2 a alespoň 80% 
        //z nich bylo větších než  5.
    
    KT=Frequencies;
    [A,B]=size(KT);
    Boundary=1:A;
    Error=%F;
          
    ///////////////Všechny >2
    M=1;
    End=%F;
    
    while ~End
        E=Teor_freq(KT);
        Smaller2=or(E(M,:)<=2);
        if A==1 then //Jen 1 řádek nebo sloupec
            End=%T;
            Error=%T;            
        elseif M==A //Jsme na posledním řádku
            if Smaller2 then
                KT(M-1,:)=KT(M-1,:)+KT(M,:);
                KT(M,:)=[];
                Boundary(M)=[];
                M=M-1;
                A=A-1;
            else
                End=%T;
            end
        else //Nejsme na posledním řádku
            if Smaller2 then
                KT(M,:)=KT(M,:)+KT(M+1,:);
                KT(M+1,:)=[];
                Boundary(M+1)=[];
                A=A-1;
            else
                M=M+1;
            end
        end
    end
    
    ///////////////80% >5

    //Kolik políček Above 5
    E=Teor_freq(KT);
    E1=E(:);
    Proc=[0,0];
    [K,L]=size(E1);
    for N=1:K
        if E1(N,1)>5 then
            Proc(1)=Proc(1)+1;
        else
            Proc(2)=Proc(2)+1;
        end;
    end;
    Proc5=Proc(1)/sum(Proc);
    End=(Proc5>=0.8);
    
    while ~End & ~Error
        [Min,k]=min(E);
        M=k(1);
        N=k(2);
        if A==1 then //Jen 1 řádek
            End=%T;
            Error=%T;
        elseif M==A //Jsme na posledním řádku
            KT(M-1,:)=KT(M-1,:)+KT(M,:);
            KT(M,:)=[];
            Boundary(M)=[];
            A=A-1;
        else //Nejsme na posledním řádku
            KT(M,:)=KT(M,:)+KT(M+1,:);
            KT(M+1,:)=[];
            Boundary(M+1)=[];
            A=A-1;   
        end
        
        //Kolik políček Above 5
        E=Teor_freq(KT);
        E1=E(:);
        Proc=[0,0];
        [K,L]=size(E1);
        for N=1:K
            if E1(N,1)>5 then
                Proc(1)=Proc(1)+1;
            else
                Proc(2)=Proc(2)+1;
            end;
        end;
        Proc5=Proc(1)/sum(Proc);
        End=(Proc5>=0.8) | A==1;
    end
    
    if A==1 then
        Error=%T;
    end
    
    if Error then
        disp('By merging rows, you could not create a pivot table to have at least two rows, all expected theoretical frequencies were greater than 2, and at least 80% of them were larger than 5. You can try to merge some columns.');
    end
    

endfunction


//Slučování řádků i sloupců
function [KT,Error]=merging_tab(Frequencies)
    //Příprava na Chí-kvadrát test 
    //Slučování řádků i sloupců tak, aby všechny očekávané teoretické četnosti byly
    //větší než 2
    //a aspoň 80% z očekávaných teoretických hodnot bylo větší než 5.
    //
    //Frequencies...Původní podrobná kontingenční tabulka
    //KT.........Výsledná sloučená kontingenční tabulka
    //Error .....Logická proměnná. Když je True, nepodařilo se vytvořit 
        //kontingenční tabulku tak, aby měla aspoň dva řádky a dva sloupce,
        //všechny očekávané teoretické četnosti byly větší než 2 
        //a alespoň 80% z nich bylo větších než  5.
    
    KT=Frequencies;
    Error=%F;
    End=%F;

    while ~End
    
        //Pokud je tabulka malá, End a Error
        [A,B]=size(KT);
        if A==1 | B==1 then
            Error=%T;
            End=%T;
        else        
            //Test, zda je KT hotová
            E=Teor_freq(KT);
            Smaller2=or(E<=2);
            Smaller5=%F;
            if ~Smaller2 then
                E1=E(:);
                Proc=[0,0];
                [K,L]=size(E1);
                for N=1:K
                    if E1(N,1)>5 then
                        Proc(1)=Proc(1)+1;
                    else
                        Proc(2)=Proc(2)+1;
                    end;
                end;
                Proc5=Proc(1)/sum(Proc);
                Smaller5=(Proc5<0.8);
            end  // if ~Smaller2 then

            //Pokud je tabulka hotová, End
            if ~Smaller2 & ~Smaller5 then
                End=%T;

            //Pokud je tabulka nehotová
            else
                //Test, zda má nejmenší součet řádek nebo sloupec
                Sum_rows=sum(KT,'c');
                Sum_columns=sum(KT,'r');
                [Min_rows,Where_rows]=min(Sum_rows);
                [Min_columns,Where_columns]=min(Sum_columns);
                Rows=(Min_rows<=Min_columns); 

                //Pokud je nejmenší součet v řádku, sluč tento řádek
                if Rows then
                
                    //Pokud je to první řádek, sluč s druhým
                    if Where_rows==1 then
                        KT(1,:)=KT(1,:)+KT(2,:);
                        KT(2,:)=[];
                    
                    //Pokud je to poslední řádek, sluč s předposledním
                    elseif Where_rows==A
                        KT(A,:)=KT(A,:)+KT(A-1,:);
                        KT(A-1,:)=[];
                    
                    //Pokud je to vnitřní řádek, sluč s tím okolním, který má menší součet
                    else
                        //Test, jestli má menší součet řádek Above nebo pod
                        Sum_above=sum(KT(Where_rows-1,:));
                        Sum_under=sum(KT(Where_rows+1,:));
                        Above=(Sum_above<=Sum_under);
                        
                        //Slučování
                        if Above then
                            KT(Where_rows,:)=KT(Where_rows,:)+KT(Where_rows-1,:);
                            KT(Where_rows-1,:)=[];
                        else
                            KT(Where_rows,:)=KT(Where_rows,:)+KT(Where_rows+1,:);
                            KT(Where_rows+1,:)=[];
                        end
                    
                    end // if Where_rows==1 then
                
                //Pokud je nejmenší součet ve sloupci, sluč tento sloupec
                else
                    //Pokud je to první sloupec, sluč s druhým
                    if Where_columns==1 then
                        KT(:,1)=KT(:,1)+KT(:,2);
                        KT(:,2)=[];
                    
                    //Pokud je to poslední sloupec, sluč s předposledním
                    elseif Where_columns==B
                        KT(:,B)=KT(:,B)+KT(:,B-1);
                        KT(:,B-1)=[];
                    
                    //Pokud je to vnitřní sloupec, sluč s tím okolním, který má menší součet
                    else
                        //Test, jestli má menší součet sloupec Left nebo vpravo
                        Sum_left=sum(KT(:,Where_columns-1));
                        Sum_right=sum(KT(:,Where_columns+1));
                        Left=(Sum_left<=Sum_right);
                        
                        //Slučování
                        if Left then
                            KT(:,Where_columns)=KT(:,Where_columns)+KT(:,Where_columns-1);
                            KT(:,Where_columns-1)=[];
                        else
                            KT(:,Where_columns)=KT(:,Where_columns)+KT(:,Where_columns+1);
                            KT(:,Where_columns+1)=[];
                        end
                    end // if Where_columns==1 then
                end   // if Rows then
            end    // if ~Smaller2 & ~Smaller5 then
        end    // if A==1 | B==1 then
    end    //while

    if Error then
        disp('By merging rows and columns, you could not create a pivot table to have at least two rows and two columns, all expected theoretical frequencies were greater than 2, and at least 80% of them were larger than 5.');
    end
    
endfunction

// Tests -------------------------------------------------------------------------
function pv=chisquare_test_i(KT)
    //chi square test Ef independence
    //pv ... p-value
    //KT ... cEntingency tabulka
    //alpha ... cEnfidence
    [KT,Error]=merging_tab(KT);
    if ~Error then        
        [r,s]=size(KT);
        pr_KT=KT/sum(KT);
        pr_E=sum(pr_KT,2)*sum(pr_KT,1);
        E=pr_E*sum(KT);
        
        T=sum(((KT-E).^2) ./E);  
        Df=(r-1)*(s-1);
        [P,Q]=cdfchi("PQ",T,Df)
        pv=Q;
    else
        pv=%nan;
    end;
endfunction



//chi square tests
function pv=chisquare_test(O,E)
    //chi square test of homogeneity
    //pv ... p-value
    //O ... observed frequencies
    //E ... expected frequencies
    //alpha ... confidence
    O=O(:)';
    E=E(:)';
    n=size(O,2);
    
    T=sum((O-E).^2 ./E);
    [P,Q]=cdfchi("PQ",T,n-1)
    pv=Q;
endfunction


//chi square tests of equivalence of two discrete discribution
function pv=chisquare_test_2(x1,x2)
    //chi square tests of equivalence of two discrete discribution
    //pv ... p-value
    
    x1=x1(:);
    x2=x2(:);
    
    //Which values are in the data?
    x=[x1;x2];
    x=gsort(x,'g','i');
    for n=length(x):-1:2
        if x(n)==x(n-1) then
            x(n)=[];
        end;
    end;
    value=x;
    
    //Which are the frequency of the values?
    for n=1:length(value)
        Tab(n,1)=length(x1(x1==value(n)));
        Tab(n,2)=length(x2(x2==value(n)));
    end
    
    //Merging of rows.
    [O,Boundary,Error]=merging_rows(Tab);
    if Error then
        pv=%nan;
    else
        E=Teor_freq(O);
        O=O(:)';
        E=E(:)';
        n=length(O)/2;
        
        T=sum((O-E).^2 ./E);
        [P,Q]=cdfchi("PQ",T,n-1)
        pv=Q;        
    end;    
endfunction

//chi square tests of equivalence of two discrete discribution
function pv=chisquare_test_n(L)
    //chi square tests of equivalence of two discrete discribution
    //pv ... p-value
    
    x=[];
    for n=1:length(n);
        xn=L(n);
        xn=xn(:);
        x=[x;xn];
    end
    
    //Which values are in the data?
    x=gsort(x,'g','i');
    for n=length(x):-1:2
        if x(n)==x(n-1) then
            x(n)=[];
        end;
    end;
    value=x;
    
    //Which are the frequency of the values?
    for n=1:length(value)
        for k=1:length(L)
            xn=L(k);
            xn=xn(:);
            Tab(n,k)=length(xn(xn==value(n)));
        end;
    end;
    
    //Merging of rows.
    [O,Boundary,Error]=merging_rows(Tab);
    if Error then
        pv=%nan;
    else
        E=Teor_freq(O);
        [n,k]=size(E);
        O=O(:)';
        E=E(:)';
        
        T=sum((O-E).^2 ./E);
        [P,Q]=cdfchi("PQ",T,(n-1)*(k-1))
        pv=Q;        
    end;    
endfunction

// test bělosti rezidui
function pv=wz_test(e)
    //test of order independence of residues
    //pv... p-value
    //e ... residues
    e=e(:)';
    me_e=median(e);
    rozdil=e-me_e;
    b=1;
    rozdil_bez_0=rozdil(rozdil~=0);
    n=size(rozdil_bez_0,2);
    
    for i=2:n
        if rozdil_bez_0(i-1)*rozdil_bez_0(i)<0 then
            b=b+1;
        end
    end
    pv=cdfbin("PQ",b-1,n-1,0.5,0.5);
endfunction

// znaménkový test
function pv=sign_test(X,Y,side)
    //Sign test
    //pv ... p-value
    //X ... median or first sample
    //Y ... second sample
    //side ... typ IS ('l'=left, 'r'=right, 'b' both)

    if (size(X)==size(Y) | size(X)==[1,1] | size(Y)==[1,1]) & (side=='l' | side=='r' | side=='b') then
        S=sign(X-Y);
        S(S==0)=[];
        n=length(S);
        Splus=S(S==-1);
        m=length(Splus);
        if side=='l' then
            pv=cdfbin("PQ",m,n,0.5,0.5);
        elseif side=='r' then
            pv=1-cdfbin("PQ",max(m-1,0),n,0.5,0.5)*sign(m);
        else
            pv=min(1,2*min(cdfbin("PQ",m,n,0.5,0.5),1-cdfbin("PQ",max(m-1,0),n,0.5,0.5)*sign(m)));
        end;   
    elseif (side=='l' | side=='r' | side=='b') then
        disp('Data X and Y dimensions are not compatible!');  
        pv=%nan;      
    else
        disp('Parameter side is out of range!');  
        pv=%nan;             
    end;
endfunction

// test podílu
function pv=prop_testp(p0,p,n,side)
    //Prop test
    //pv... p-value
    //p0 ... probab. via H0
    //p ... probab. from sample
    //n ... sample length
    //side ... typ IS ('l'=left, 'r'=right, 'b' both)
    if p0>0 & p0<1 & p>=0 & p<=1 & n>0 & n==fix(n) & (side=='l' | side=='r' | side=='b') then
        m=round(p*n);
        if side=='l' then
            pv=cdfbin("PQ",m,n,p0,1-p0);
        elseif side=='r' then
            pv=1-cdfbin("PQ",max(m-1,0),n,p0,1-p0)*sign(m);
        else
            pv=min(1,2*min(cdfbin("PQ",m,n,p0,1-p0),1-cdfbin("PQ",max(m-1,0),n,p0,1-p0)*sign(m)));
        end;   
    else
        disp('Input parameter is out of range!');  
        pv=%nan;      
    end;
endfunction
